@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 """
-besapi.py
+This module besapi provides a simple interface to the BigFix REST API.
 
 MIT License
 Copyright (c) 2014 Matt Hansen
@@ -12,28 +12,28 @@ Library for communicating with the BES (BigFix) REST API.
 import configparser
 import datetime
 import hashlib
+import importlib.resources
 import json
 import logging
 import os
 import random
 import site
 import string
+import urllib.parse
 
-try:
-    from urllib import parse
-except ImportError:
-    from urlparse import parse_qs as parse
-
+import lxml.etree
+import lxml.objectify
 import requests
-from lxml import etree, objectify
-from pkg_resources import resource_filename
 
-logging.basicConfig(level=logging.WARNING)
+__version__ = "3.9.4"
+
 besapi_logger = logging.getLogger("besapi")
 
+# pylint: disable=consider-using-f-string
+
 
 def rand_password(length=20):
-    """get a random password"""
+    """Get a random password."""
 
     all_safe_chars = string.ascii_letters + string.digits + "!#()*+,-.:;<=>?[]^_|~"
 
@@ -44,7 +44,7 @@ def rand_password(length=20):
 
 def sanitize_txt(*args):
     """Clean arbitrary text for safe file system usage."""
-    valid_chars = "-_.() %s%s" % (string.ascii_letters, string.digits)
+    valid_chars = f"-_.() {string.ascii_letters}{string.digits}"
 
     sani_args = []
     for arg in args:
@@ -64,6 +64,7 @@ def sanitize_txt(*args):
 def elem2dict(node):
     """
     Convert an lxml.etree node tree into a dict.
+
     https://gist.github.com/jacobian/795571?permalink_comment_id=2981870#gistcomment-2981870
     """
     result = {}
@@ -92,7 +93,10 @@ def elem2dict(node):
 def replace_text_between(
     original_text, first_delimiter, second_delimiter, replacement_text
 ):
-    """Replace text between delimeters. Each delimiter should only appear once."""
+    """Replace text between delimiters.
+
+    Each delimiter should only appear once.
+    """
     leading_text = original_text.split(first_delimiter)[0]
     trailing_text = original_text.split(second_delimiter)[1]
 
@@ -107,7 +111,7 @@ def replace_text_between(
 
 # https://github.com/jgstew/generate_bes_from_template/blob/bcc6c79632dd375c2861608ded3ae5872801a669/src/generate_bes_from_template/generate_bes_from_template.py#L87-L92
 def parse_bes_modtime(string_datetime):
-    """parse datetime string to object"""
+    """Parse datetime string to object."""
     # ("%a, %d %b %Y %H:%M:%S %z")
     return datetime.datetime.strptime(string_datetime, "%a, %d %b %Y %H:%M:%S %z")
 
@@ -118,7 +122,7 @@ def parse_bes_modtime(string_datetime):
 #     """custom HTTPAdapter for requests to override blocksize
 #     for Uploading or Downloading large files"""
 
-#     # override inti_poolmanager from regular HTTPAdapter
+#     # override init_poolmanager from regular HTTPAdapter
 #     # https://stackoverflow.com/questions/22915295/python-requests-post-and-big-content/22915488#comment125583017_22915488
 #     def init_poolmanager(self, connections, maxsize, block=False, **pool_kwargs):
 #         """Initializes a urllib3 PoolManager.
@@ -150,9 +154,153 @@ def parse_bes_modtime(string_datetime):
 #         )
 
 
+def get_action_combined_relevance(relevances: list[str]):
+    """Take array of ordered relevance clauses and return relevance string for
+    action.
+    """
+
+    relevance_combined = ""
+
+    if not relevances:
+        return "False"
+    if len(relevances) == 0:
+        return "False"
+    if len(relevances) == 1:
+        return relevances[0]
+    if len(relevances) > 0:
+        for clause in relevances:
+            if len(relevance_combined) == 0:
+                relevance_combined = clause
+            else:
+                relevance_combined = (
+                    "( " + relevance_combined + " ) AND ( " + clause + " )"
+                )
+
+    return relevance_combined
+
+
+def get_target_xml(targets=None):
+    """Get target xml based upon input.
+
+    Input can be a single string:
+        - starts with "<AllComputers>" if all computers should be targeted
+        - Otherwise will be interpreted as custom relevance
+
+    Input can be a single int:
+        - Single Computer ID Target
+
+    Input can be an array:
+        - Array of Strings: ComputerName
+        - Array of Integers: ComputerID
+    """
+    if targets is None or not targets:
+        logging.warning("No valid targeting found, will target no computers.")
+        # default if invalid:
+        return "<CustomRelevance>False</CustomRelevance>"
+
+    # if targets is int:
+    if isinstance(targets, int):
+        if targets == 0:
+            raise ValueError(
+                "Int 0 is not valid Computer ID, set targets to an array of strings of computer names or an array of ints of computer ids or custom relevance string or <AllComputers>"
+            )
+        return f"<ComputerID>{targets}</ComputerID>"
+
+    # if targets is str:
+    if isinstance(targets, str):
+        # if targets string starts with "<AllComputers>":
+        if targets.startswith("<AllComputers>"):
+            if "false" in targets.lower():
+                # In my testing, <AllComputers>false</AllComputers> does not work correctly
+                return "<CustomRelevance>False</CustomRelevance>"
+                # return "<AllComputers>false</AllComputers>"
+            return "<AllComputers>true</AllComputers>"
+        # treat as custom relevance:
+        return f"<CustomRelevance><![CDATA[{targets}]]></CustomRelevance>"
+
+    # if targets is array:
+    if isinstance(targets, list):
+        element_type = type(targets[0])
+        if element_type is int:
+            # array of computer ids
+            return (
+                "<ComputerID>"
+                + "</ComputerID><ComputerID>".join(map(str, targets))
+                + "</ComputerID>"
+            )
+        if element_type is str:
+            # array of computer names
+            return (
+                "<ComputerName>"
+                + "</ComputerName><ComputerName>".join(targets)
+                + "</ComputerName>"
+            )
+
+    logging.warning("No valid targeting found, will target no computers.")
+
+    # default if invalid:
+    return "<CustomRelevance>False</CustomRelevance>"
+
+
+def validate_xsd(doc):
+    """Validate results using XML XSDs."""
+    try:
+        xmldoc = lxml.etree.fromstring(doc)
+    except BaseException:  # pylint: disable=broad-except
+        return False
+
+    for xsd in ["BES.xsd", "BESAPI.xsd", "BESActionSettings.xsd"]:
+        schema_path = importlib.resources.files(__package__) / f"schemas/{xsd}"
+        with schema_path.open("r") as xsd_file:
+            xmlschema_doc = lxml.etree.parse(xsd_file)
+
+        # one schema may throw an error while another will validate
+        try:
+            xmlschema = lxml.etree.XMLSchema(xmlschema_doc)
+        except lxml.etree.XMLSchemaParseError as err:
+            # this should only error if the XSD itself is malformed
+            besapi_logger.error("ERROR with `%s`: %s", xsd, err)
+            raise err
+
+        if xmlschema.validate(xmldoc):
+            return True
+
+    return False
+
+
+def validate_xml_bes_file(file_path):
+    """Take a file path as input,
+    read as binary data,.
+
+    validate against xml schema
+
+    returns True for valid xml
+    returns False for invalid xml (or if file is not xml)
+    """
+    with open(file_path, "rb") as file:
+        file_data = file.read()
+
+    return validate_xsd(file_data)
+
+
+def get_bes_conn_using_env():
+    """Get BESConnection using environment variables."""
+    username = os.getenv("BES_USER_NAME")
+    password = os.getenv("BES_PASSWORD")
+    rootserver = os.getenv("BES_ROOT_SERVER")
+
+    if username and password and rootserver:
+        bes_conn = BESConnection(username, password, rootserver)
+        if bes_conn:
+            return bes_conn
+
+    return None
+
+
 def get_bes_conn_using_config_file(conf_file=None):
     """
-    read connection values from config file
+    Read connection values from config file.
+
     return besapi connection
     """
     config_paths = [
@@ -173,17 +321,17 @@ def get_bes_conn_using_config_file(conf_file=None):
         print("Attempting BESAPI Connection using config file:", found_config_files)
         try:
             BES_ROOT_SERVER = configparser_instance.get("besapi", "BES_ROOT_SERVER")
-        except BaseException:
+        except BaseException:  # pylint: disable=broad-except
             BES_ROOT_SERVER = None
 
         try:
             BES_USER_NAME = configparser_instance.get("besapi", "BES_USER_NAME")
-        except BaseException:
+        except BaseException:  # pylint: disable=broad-except
             BES_USER_NAME = None
 
         try:
             BES_PASSWORD = configparser_instance.get("besapi", "BES_PASSWORD")
-        except BaseException:
+        except BaseException:  # pylint: disable=broad-except
             BES_PASSWORD = None
 
         if BES_ROOT_SERVER and BES_USER_NAME and BES_PASSWORD:
@@ -193,7 +341,7 @@ def get_bes_conn_using_config_file(conf_file=None):
 
 
 class BESConnection:
-    """BigFix RESTAPI connection abstraction class"""
+    """BigFix RESTAPI connection abstraction class."""
 
     def __init__(self, username, password, rootserver, verify=False):
         if not verify:
@@ -222,16 +370,16 @@ class BESConnection:
         try:
             # get root server port
             self.rootserver_port = int(rootserver.split("://", 1)[1].split(":", 1)[1])
-        except BaseException:
+        except BaseException:  # pylint: disable=broad-except
             # if error, assume default
             self.rootserver_port = 52311
 
         self.login()
 
     def __repr__(self):
-        """object representation"""
+        """Object representation."""
         # https://stackoverflow.com/a/2626364/861745
-        return f"Object: besapi.BESConnction( username={self.username}, rootserver={self.rootserver} )"
+        return f"Object: besapi.BESConnection( username={self.username}, rootserver={self.rootserver} )"
 
     def __eq__(self, other):
         if (
@@ -243,65 +391,128 @@ class BESConnection:
         return False
 
     def __del__(self):
-        """cleanup on deletion of instance"""
+        """Cleanup on deletion of instance."""
         self.logout()
         self.session.auth = None
 
     def __bool__(self):
-        """get true or false"""
+        """Get true or false."""
         return self.login()
 
     def url(self, path):
-        """get absolute url"""
+        """Get absolute url."""
         if path.startswith(self.rootserver):
             url = path
         else:
-            url = "%s/api/%s" % (self.rootserver, path)
+            url = f"{self.rootserver}/api/{path}"
 
         return url
 
     def get(self, path="help", **kwargs):
-        """HTTP GET request"""
+        """HTTP GET request."""
         self.last_connected = datetime.datetime.now()
         return RESTResult(
             self.session.get(self.url(path), verify=self.verify, **kwargs)
         )
 
-    def post(self, path, data, **kwargs):
-        """HTTP POST request"""
+    def post(self, path, data, validate_xml=None, **kwargs):
+        """HTTP POST request."""
+
+        # if validate_xml is true, data must validate to xml schema
+        # if validate_xml is false, no schema check will be made
+        if validate_xml:
+            if not validate_xsd(data):
+                err_msg = "data being posted did not validate to XML schema. If expected, consider setting validate_xml to false."
+                if validate_xml:
+                    logging.error(err_msg)
+                    raise ValueError(err_msg)
+
+                # this is intended it validate_xml is None, but not used currently
+                logging.warning(err_msg)
+
         self.last_connected = datetime.datetime.now()
         return RESTResult(
             self.session.post(self.url(path), data=data, verify=self.verify, **kwargs)
         )
 
-    def put(self, path, data, **kwargs):
-        """HTTP PUT request"""
+    def put(self, path, data, validate_xml=None, **kwargs):
+        """HTTP PUT request."""
         self.last_connected = datetime.datetime.now()
+
+        # if validate_xml is true, data must validate to xml schema
+        # if validate_xml is false, no schema check will be made
+        if validate_xml:
+            if not validate_xsd(data):
+                err_msg = "data being put did not validate to XML schema. If expected, consider setting validate_xml to false."
+                if validate_xml:
+                    logging.error(err_msg)
+                    raise ValueError(err_msg)
+
+                # this is intended it validate_xml is None, but not used currently
+                logging.warning(err_msg)
+
         return RESTResult(
             self.session.put(self.url(path), data=data, verify=self.verify, **kwargs)
         )
 
     def delete(self, path, **kwargs):
-        """HTTP DELETE request"""
+        """HTTP DELETE request."""
         self.last_connected = datetime.datetime.now()
         return RESTResult(
             self.session.delete(self.url(path), verify=self.verify, **kwargs)
         )
 
+    def session_relevance_json(self, relevance, **kwargs):
+        """Get Session Relevance Results in JSON.
+
+        This will submit the relevance string as json instead of html form data.
+        """
+        session_relevance = urllib.parse.quote(relevance, safe=":+")
+        rel_data = {"output": "json", "relevance": session_relevance}
+        self.last_connected = datetime.datetime.now()
+        result = RESTResult(
+            self.session.post(
+                self.url("query"),
+                data=rel_data,
+                verify=self.verify,
+                **kwargs,
+            )
+        )
+        return json.loads(result.text)
+
+    def session_relevance_json_array(self, relevance, **kwargs):
+        """Get Session Relevance Results in an array from the json return.
+
+        This will submit the relevance string as json instead of html form data.
+        """
+        result = self.session_relevance_json(relevance, **kwargs)
+        return result["result"]
+
+    def session_relevance_json_string(self, relevance, **kwargs):
+        """Get Session Relevance Results in a string from the json return.
+
+        This will submit the relevance string as json instead of html form data.
+        """
+        # not sure if the following is needed to handle some cases:
+        # relevance = "(it as string) of ( " + relevance + " )"
+        rel_result_array = self.session_relevance_json_array(relevance, **kwargs)
+        # Ensure each element is converted to a string
+        return "\n".join(map(str, rel_result_array))
+
     def session_relevance_xml(self, relevance, **kwargs):
-        """Get Session Relevance Results XML"""
+        """Get Session Relevance Results XML."""
         self.last_connected = datetime.datetime.now()
         return RESTResult(
             self.session.post(
                 self.url("query"),
-                data=f"relevance={parse.quote(relevance, safe=':+')}",
+                data=f"relevance={urllib.parse.quote(relevance, safe=':+')}",
                 verify=self.verify,
                 **kwargs,
             )
         )
 
     def session_relevance_array(self, relevance, **kwargs):
-        """Get Session Relevance Results array"""
+        """Get Session Relevance Results array."""
         rel_result = self.session_relevance_xml(relevance, **kwargs)
         # print(rel_result)
         result = []
@@ -313,24 +524,29 @@ class BESConnection:
             if "no such child: Answer" in str(err):
                 try:
                     result.append("ERROR: " + rel_result.besobj.Query.Error.text)
-                except AttributeError as err:
-                    if "no such child: Error" in str(err):
+                except AttributeError as err2:
+                    if "no such child: Error" in str(err2):
                         result.append("<Nothing> Nothing returned, but no error.")
                         besapi_logger.info("Query did not return any results")
                     else:
-                        besapi_logger.error("%s\n%s", err, rel_result.text)
+                        besapi_logger.error("%s\n%s", err2, rel_result.text)
+                        result.append("ERROR: " + rel_result.text)
                         raise
             else:
+                besapi_logger.error("%s\n%s", err, rel_result.text)
+                result.append("ERROR: " + rel_result.text)
                 raise
         return result
 
     def session_relevance_string(self, relevance, **kwargs):
-        """Get Session Relevance Results string"""
-        rel_result_array = self.session_relevance_array(relevance, **kwargs)
+        """Get Session Relevance Results string."""
+        rel_result_array = self.session_relevance_array(
+            "(it as string) of ( " + relevance + " )", **kwargs
+        )
         return "\n".join(rel_result_array)
 
     def login(self):
-        """do login"""
+        """Do login."""
         if bool(self.last_connected):
             duration_obj = datetime.datetime.now() - self.last_connected
             duration_minutes = duration_obj / datetime.timedelta(minutes=1)
@@ -361,12 +577,40 @@ class BESConnection:
         return bool(self.last_connected)
 
     def logout(self):
-        """clear session and close it"""
+        """Clear session and close it."""
         self.session.cookies.clear()
         self.session.close()
 
+    def set_dashboard_variable_value(
+        self, dashboard_name, var_name, var_value, private=False
+    ):
+        """Set the variable value from a dashboard datastore."""
+
+        dash_var_xml = f"""<BESAPI xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="BESAPI.xsd">
+            <DashboardData>
+                    <Dashboard>{dashboard_name}</Dashboard>
+                    <Name>{var_name}</Name>
+                    <IsPrivate>{str(private).lower()}</IsPrivate>
+                    <Value>{var_value}</Value>
+            </DashboardData>
+    </BESAPI>
+    """
+
+        return self.post(
+            f"dashboardvariable/{dashboard_name}/{var_name}", data=dash_var_xml
+        )
+
+    def get_dashboard_variable_value(self, dashboard_name, var_name):
+        """Get the variable value from a dashboard datastore."""
+
+        return str(
+            self.get(
+                f"dashboardvariable/{dashboard_name}/{var_name}"
+            ).besobj.DashboardData.Value
+        )
+
     def validate_site_path(self, site_path, check_site_exists=True, raise_error=False):
-        """make sure site_path is valid"""
+        """Make sure site_path is valid."""
 
         if site_path is None:
             if not raise_error:
@@ -390,18 +634,18 @@ class BESConnection:
                 if not check_site_exists:
                     # don't check if site exists first
                     return site_path
-                else:
-                    # check site exists first
-                    site_result = self.get(f"site/{site_path}")
-                    if site_result.request.status_code != 200:
-                        besapi_logger.info("Site `%s` does not exist", site_path)
-                        if not raise_error:
-                            return None
 
-                        raise ValueError(f"Site at path `{site_path}` does not exist!")
+                # check site exists first
+                site_result = self.get(f"site/{site_path}")
+                if site_result.request.status_code != 200:
+                    besapi_logger.info("Site `%s` does not exist", site_path)
+                    if not raise_error:
+                        return None
 
-                    # site_path is valid and exists:
-                    return site_path
+                    raise ValueError(f"Site at path `{site_path}` does not exist!")
+
+                # site_path is valid and exists:
+                return site_path
 
         # Invalid: No valid prefix found
         raise ValueError(
@@ -409,8 +653,9 @@ class BESConnection:
         )
 
     def get_current_site_path(self, site_path=None):
-        """if site_path is none, get current instance site_path,
-        otherwise validate and return provided site_path"""
+        """If site_path is none, get current instance site_path,
+        otherwise validate and return provided site_path.
+        """
 
         # use instance site_path context if none provided:
         if site_path is None or str(site_path).strip() == "":
@@ -424,15 +669,40 @@ class BESConnection:
         return self.validate_site_path(site_path, check_site_exists=False)
 
     def set_current_site_path(self, site_path):
-        """set current site path context"""
+        """Set current site path context."""
 
         if self.validate_site_path(site_path):
             self.site_path = site_path
             return self.site_path
 
+        return None
+
+    def import_bes_to_site(self, bes_file_path, site_path=None):
+        """Import bes file to site."""
+
+        if not os.access(bes_file_path, os.R_OK):
+            besapi_logger.error("%s is not readable", bes_file_path)
+            raise FileNotFoundError(f"{bes_file_path} is not readable")
+
+        site_path = self.get_current_site_path(site_path)
+
+        self.validate_site_path(site_path, False, True)
+
+        with open(bes_file_path, "rb") as f:
+            content = f.read()
+
+            # validate BES File contents:
+            if not validate_xsd(content):
+                besapi_logger.error("%s is not valid", bes_file_path)
+                return None
+
+            # https://developer.bigfix.com/rest-api/api/import.html
+            result = self.post(f"import/{site_path}", content)
+            return result
+
     def create_site_from_file(self, bes_file_path, site_type="custom"):
-        """create new site"""
-        xml_parsed = etree.parse(bes_file_path)
+        """Create new site."""
+        xml_parsed = lxml.etree.parse(bes_file_path)
         new_site_name = xml_parsed.xpath("/BES/CustomSite/Name/text()")[0]
 
         result_site_path = self.validate_site_path(
@@ -443,12 +713,12 @@ class BESConnection:
             besapi_logger.warning("Site `%s` already exists", result_site_path)
             return None
 
-        result_site = self.post("sites", etree.tostring(xml_parsed))
+        result_site = self.post("sites", lxml.etree.tostring(xml_parsed))
 
         return result_site
 
     def get_user(self, user_name):
-        """get a user"""
+        """Get a user."""
 
         result_users = self.get(f"operator/{user_name}")
 
@@ -458,8 +728,8 @@ class BESConnection:
         besapi_logger.info("User `%s` Not Found!", user_name)
 
     def create_user_from_file(self, bes_file_path):
-        """create user from xml"""
-        xml_parsed = etree.parse(bes_file_path)
+        """Create user from xml."""
+        xml_parsed = lxml.etree.parse(bes_file_path)
         new_user_name = xml_parsed.xpath("/BESAPI/Operator/Name/text()")[0]
         result_user = self.get_user(new_user_name)
 
@@ -467,12 +737,12 @@ class BESConnection:
             besapi_logger.warning("User `%s` Already Exists!", new_user_name)
             return result_user
         besapi_logger.info("Creating User `%s`", new_user_name)
-        _ = self.post("operators", etree.tostring(xml_parsed))
+        _ = self.post("operators", lxml.etree.tostring(xml_parsed))
         # print(user_result)
         return self.get_user(new_user_name)
 
     def get_computergroup(self, group_name, site_path=None):
-        """get computer group resource URI"""
+        """Get computer group resource URI."""
 
         site_path = self.get_current_site_path(site_path)
         result_groups = self.get(f"computergroups/{site_path}")
@@ -487,9 +757,9 @@ class BESConnection:
         besapi_logger.info("Group `%s` Not Found!", group_name)
 
     def create_group_from_file(self, bes_file_path, site_path=None):
-        """create a new group"""
+        """Create a new group."""
         site_path = self.get_current_site_path(site_path)
-        xml_parsed = etree.parse(bes_file_path)
+        xml_parsed = lxml.etree.parse(bes_file_path)
         new_group_name = xml_parsed.xpath("/BES/ComputerGroup/Title/text()")[0]
 
         existing_group = self.get_computergroup(new_group_name, site_path)
@@ -500,13 +770,13 @@ class BESConnection:
 
         # print(lxml.etree.tostring(xml_parsed))
 
-        _ = self.post(f"computergroups/{site_path}", etree.tostring(xml_parsed))
+        _ = self.post(f"computergroups/{site_path}", lxml.etree.tostring(xml_parsed))
 
         return self.get_computergroup(site_path, new_group_name)
 
     def get_upload(self, file_name, file_hash):
         """
-        check for a specific file upload reference
+        Check for a specific file upload reference.
 
         each upload is uniquely identified by sha1 and filename
 
@@ -532,7 +802,8 @@ class BESConnection:
 
     def upload(self, file_path, file_name=None, file_hash=None):
         """
-        upload a single file
+        Upload a single file.
+
         https://developer.bigfix.com/rest-api/api/upload.html
         """
         if not os.access(file_path, os.R_OK):
@@ -543,6 +814,13 @@ class BESConnection:
         if not file_name:
             file_name = os.path.basename(file_path)
 
+        # files cannot contain spaces:
+        if " " in file_name:
+            besapi_logger.warning(
+                "Replacing spaces with underscores in `%s`", file_name
+            )
+            file_name = file_name.replace(" ", "_")
+
         if not file_hash:
             besapi_logger.warning(
                 "SHA1 hash of file to be uploaded not provided, calculating it."
@@ -559,10 +837,7 @@ class BESConnection:
 
         check_upload = None
         if file_hash:
-            try:
-                check_upload = self.get_upload(str(file_name), str(file_hash))
-            except BaseException as err:
-                print(err)
+            check_upload = self.get_upload(str(file_name), str(file_hash))
 
             if check_upload:
                 besapi_logger.warning(
@@ -579,11 +854,11 @@ class BESConnection:
     def parse_upload_result_to_prefetch(
         self, result_upload, use_localhost=True, use_https=True
     ):
-        """take a rest response from an upload and parse into prefetch"""
+        """Take a rest response from an upload and parse into prefetch."""
         file_url = str(result_upload.besobj.FileUpload.URL)
         if use_https:
             file_url = file_url.replace("http://", "https://")
-        # there are 3 different posibilities for the server FQDN
+        # there are 3 different possibilities for the server FQDN
         # localhost
         # self.rootserver (without port number)
         # the returned value from the upload result
@@ -600,7 +875,7 @@ class BESConnection:
         return f"prefetch {file_name} sha1:{file_sha1} size:{file_size} {file_url} sha256:{file_sha256}"
 
     def get_content_by_resource(self, resource_url):
-        """get a single content item by resource"""
+        """Get a single content item by resource."""
         # Get Specific Content
         content = None
         try:
@@ -618,9 +893,16 @@ class BESConnection:
         return content
 
     def update_item_from_file(self, file_path, site_path=None):
-        """update an item by name and last modified"""
+        """Update an item by name and last modified."""
         site_path = self.get_current_site_path(site_path)
-        bes_tree = etree.parse(file_path)
+        bes_tree = lxml.etree.parse(file_path)
+
+        with open(file_path, "rb") as f:
+            content = f.read()
+            if not validate_xsd(content):
+                besapi_logger.error("%s is not valid", file_path)
+                return None
+
         # get name of first child tag of BES
         # - https://stackoverflow.com/a/3601919/861745
         bes_type = str(bes_tree.xpath("name(/BES/*[1])"))
@@ -644,7 +926,7 @@ class BESConnection:
         export_folder="./",
         name_trim=100,
     ):
-        """save an xml string to bes file"""
+        """Save an xml string to bes file."""
         item_folder = export_folder
         if not os.path.exists(item_folder):
             os.makedirs(item_folder)
@@ -672,8 +954,10 @@ class BESConnection:
         include_item_type_folder=False,
         include_item_id=False,
     ):
-        """export a single item by resource
+        """Export a single item by resource.
+
         example resources:
+
          - content_type/site_type/site/id
          - https://localhost:52311/api/content_type/site_type/site/id
         """
@@ -720,8 +1004,10 @@ class BESConnection:
         include_site_folder=True,
         include_item_ids=True,
     ):
-        """export contents of site
+        """Export contents of site.
+
         Originally here:
+
         - https://gist.github.com/jgstew/1b2da12af59b71c9f88a
         - https://bigfix.me/fixlet/details/21282
         """
@@ -797,7 +1083,7 @@ class BESConnection:
     def export_all_sites(
         self, include_external=False, export_folder="./", name_trim=70, verbose=False
     ):
-        """export all bigfix sites to a folder"""
+        """Export all bigfix sites to a folder."""
         results_sites = self.get("sites")
         if verbose:
             print(results_sites)
@@ -816,11 +1102,12 @@ class BESConnection:
 
 
 class RESTResult:
-    """BigFix REST API Result Abstraction Class"""
+    """BigFix REST API Result Abstraction Class."""
 
     def __init__(self, request):
         self.request = request
         self.text = request.text
+        self.valid = None
         self._besxml = None
         self._besobj = None
         self._besdict = None
@@ -834,7 +1121,7 @@ class RESTResult:
                     f"\n - HTTP Response Status Code: `403` Forbidden\n - ERROR: `{self.text}`\n - URL: `{self.request.url}`"
                 )
 
-            besapi_logger.info(
+            besapi_logger.debug(
                 "HTTP Request Status Code `%d` from URL `%s`",
                 self.request.status_code,
                 self.request.url,
@@ -855,7 +1142,9 @@ class RESTResult:
             if self.validate_xsd(request.text):
                 self.valid = True
             else:
-                # print("WARNING: response appears invalid")
+                logging.debug(
+                    "INFO: REST API Result does not appear to be XML, this could be expected."
+                )
                 self.valid = False
 
     def __str__(self):
@@ -863,7 +1152,7 @@ class RESTResult:
             # I think this is needed for python3 compatibility:
             try:
                 return self.besxml.decode("utf-8")
-            except BaseException:
+            except BaseException:  # pylint: disable=broad-except
                 return self.besxml
         else:
             return self.text
@@ -873,7 +1162,7 @@ class RESTResult:
 
     @property
     def besxml(self):
-        """property for parsed xml representation"""
+        """Property for parsed xml representation."""
         if self.valid and self._besxml is None:
             self._besxml = self.xmlparse_text(self.text)
 
@@ -881,7 +1170,7 @@ class RESTResult:
 
     @property
     def besobj(self):
-        """property for xml object representation"""
+        """Property for xml object representation."""
         if self.valid and self._besobj is None:
             self._besobj = self.objectify_text(self.text)
 
@@ -889,10 +1178,10 @@ class RESTResult:
 
     @property
     def besdict(self):
-        """property for python dict representation"""
+        """Property for python dict representation."""
         if self._besdict is None:
             if self.valid:
-                self._besdict = elem2dict(etree.fromstring(self.besxml))
+                self._besdict = elem2dict(lxml.etree.fromstring(self.besxml))
             else:
                 self._besdict = {"text": str(self)}
 
@@ -900,56 +1189,40 @@ class RESTResult:
 
     @property
     def besjson(self):
-        """property for json representation"""
+        """Property for json representation."""
         if self._besjson is None:
             self._besjson = json.dumps(self.besdict, indent=2)
 
         return self._besjson
 
     def validate_xsd(self, doc):
-        """validate results using XML XSDs"""
-        try:
-            xmldoc = etree.fromstring(doc)
-        except BaseException:
-            return False
-
-        for xsd in ["BES.xsd", "BESAPI.xsd", "BESActionSettings.xsd"]:
-            xmlschema_doc = etree.parse(resource_filename(__name__, "schemas/%s" % xsd))
-
-            # one schema may throw an error while another will validate
-            try:
-                xmlschema = etree.XMLSchema(xmlschema_doc)
-            except etree.XMLSchemaParseError as err:
-                # this should only error if the XSD itself is malformed
-                besapi_logger.error("ERROR with `%s`: %s", xsd, err)
-                raise err
-
-            if xmlschema.validate(xmldoc):
-                return True
-
-        return False
+        """Validate results using XML XSDs."""
+        # return self.valid if already set
+        if self.valid is not None and isinstance(self.valid, bool):
+            return self.valid
+        return validate_xsd(doc)
 
     def xmlparse_text(self, text):
-        """parse response text as xml"""
+        """Parse response text as xml."""
         if type(text) is str:
-            root_xml = etree.fromstring(text.encode("utf-8"))
+            root_xml = lxml.etree.fromstring(text.encode("utf-8"))
         else:
             root_xml = text
 
-        return etree.tostring(root_xml, encoding="utf-8", xml_declaration=True)
+        return lxml.etree.tostring(root_xml, encoding="utf-8", xml_declaration=True)
 
     def objectify_text(self, text):
-        """parse response text as objectified xml"""
+        """Parse response text as objectified xml."""
         if type(text) is str:
             root_xml = text.encode("utf-8")
         else:
             root_xml = text
 
-        return objectify.fromstring(root_xml)
+        return lxml.objectify.fromstring(root_xml)
 
 
 def main():
-    """if invoked directly, run bescli command loop"""
+    """If invoked directly, run bescli command loop."""
     # pylint: disable=import-outside-toplevel
     try:
         from bescli import bescli
@@ -960,4 +1233,5 @@ def main():
 
 
 if __name__ == "__main__":
+    logging.basicConfig()
     main()
